// -*- c++ -*-
//------------------------------------------------------------------------------
// $Id: DeckPlayer.cpp,v 1.39 2004/07/08 04:22:50 vlg Exp $
//------------------------------------------------------------------------------
//                            DeckPlayer.cpp
//------------------------------------------------------------------------------
//  Copyright (c) 2004 by Vladislav Grinchenko 
//
//  This program is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License   
//  as published by the Free Software Foundation; either version  
//  2 of the License, or (at your option) any later version.      
//------------------------------------------------------------------------------
//
// Date   : Fri Feb  6 23:37:11 EST 2004
//
//------------------------------------------------------------------------------

#include <gtkmm/stock.h>
#include <gtkmm/separator.h>
#include <gtkmm/box.h>
#include <gtkmm/handlebox.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/frame.h>
#include <gtkmm/toolbar.h>
#include <gtkmm/textattributes.h>
#include <gtkmm/main.h>
#include <gtkmm/progressbar.h>

#include "Granule-main.h"
#include "Granule.h"
#include "MainWindow.h"
#include "Config.h"
#include "Card.h"
#include "Deck.h"
#include "CardBox.h"
#include "DeckPlayer.h"
#include "DeckView.h"

#include "ButtonWithImageLabel.h"

Gtk::StockID DeckPlayer::SAY_IT ("say-it");

DeckPlayer::
DeckPlayer (VDeck& vdeck_) :
	m_close_button   (
		manage (new ButtonWithImageLabel (Gtk::Stock::CLOSE, "Close"))),
	m_shuffle_button (
		manage (new ButtonWithImageLabel (Gtk::Stock::REFRESH, "Shuffle"))),
	m_edit_button    (
		manage (new ButtonWithImageLabel (Gtk::Stock::DND, "Edit"))),
	m_selected_side  (FRONT),
	m_notebook       (manage (new Gtk::Notebook)),
	m_play_progress  (manage (new Gtk::ProgressBar)),
	m_bg_white       ("white"),
	m_bg_pink        ("pink"),
	m_bg_black       ("black"),
	m_deck           (vdeck_),
	m_deck_iter      (m_deck.end ()),         /* empty deck */
	m_marker         (Gtk::TextBuffer::Tag::create ()),
	m_is_lesson_learned (false),
	m_is_real_deck   (true)
{
	trace_with_mask("DeckPlayer::DeckPlayer", GUITRACE);

	set_title ("DeckPlayer");
	set_resizable ();
    set_size_request (583, 350); // width; height (3x5 card ratio)
    set_border_width (2);
	set_position (Gtk::WIN_POS_CENTER_ALWAYS);
	set_modal (false);

	/** Are we dealing with CardDeck?
	 */
	if (dynamic_cast<Deck*>(&m_deck) == NULL) {
		m_is_real_deck = false;
		DL((GRAPP,"Deck isn't REAL!\n"));
	}
	else {
		DL((GRAPP,"Yep, this IS a real deck\n"));
	}

	/** HBox holds both the notebook and controls/info side panel
	 */
	Gtk::HBox* hbox = manage (new Gtk::HBox (false, 2));
	hbox->pack_start (*m_notebook);

	/** Add the notebook
	 */
	m_notebook->set_tab_pos (Gtk::POS_BOTTOM);
	m_notebook->set_show_border (true);
	m_notebook->modify_bg   (Gtk::STATE_NORMAL, m_bg_white);

	/** Fill up notebook with elements; first front of the card
	 */
	m_front.set_use_markup (true);
	m_front.set_alignment (0.46, 0.5);
	m_front.set_justify (Gtk::JUSTIFY_LEFT);
	m_front.modify_font (CONFIG->get_font ());
	m_front.set_selectable ();

    Gtk::Frame* frame = manage (new Gtk::Frame());
    frame->set_label_align(0, 0);
    frame->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
	frame->modify_bg   (Gtk::STATE_NORMAL, m_bg_black);
	frame->add (m_front);

	m_notebook->pages ().push_back (
		Gtk::Notebook_Helpers::TabElem (*frame, "Front"));

	/** Back of the card
	 */
	Gtk::VBox* vbox = manage (new Gtk::VBox (false, 20));
	Gtk::HSeparator* separator = manage (new Gtk::HSeparator);
	separator->modify_bg   (Gtk::STATE_NORMAL, m_bg_pink);
	separator->modify_fg   (Gtk::STATE_NORMAL, m_bg_pink);
	separator->modify_base (Gtk::STATE_NORMAL, m_bg_pink);

	m_back.set_justify (Gtk::JUSTIFY_FILL);
	m_back.set_selectable ();

	m_example.set_wrap_mode (Gtk::WRAP_WORD);
	m_example.set_editable (false);
	m_example.set_justification (Gtk::JUSTIFY_LEFT);
	m_example.set_left_margin (4);
	m_example.set_right_margin (4);
	m_example.modify_font (CONFIG->get_font ());

	vbox->pack_start (m_back, Gtk::PACK_SHRINK, 7);
	vbox->pack_start (*separator, Gtk::PACK_SHRINK, 6);
	vbox->pack_end   (m_example, Gtk::PACK_EXPAND_WIDGET, 3);

	m_notebook->pages ().push_back (
		Gtk::Notebook_Helpers::TabElem (*vbox, "Back"));

	/** Create a highlight tag
	 */
	m_tag_table = m_example.get_buffer ()->get_tag_table ();
	m_marker->property_foreground () = "red";
	m_tag_table->add (m_marker);

	/** Side-panel controls 
	 */
	vbox = manage (new Gtk::VBox (false, 5));

    Gtk::RadioButton::Group group;
    Gtk::RadioButton* radio_fb;
    Gtk::RadioButton* radio_bb;
    Gtk::VButtonBox* vbuttonbox = manage (new Gtk::VButtonBox);
    vbuttonbox->set_homogeneous ();
    vbuttonbox->set_spacing (1);
                                                                                
    radio_fb = manage (new Gtk::RadioButton (group, "Ask front"));
    radio_fb->signal_clicked ().connect (
        sigc::bind<SideSelection>(
            sigc::mem_fun (*this, &DeckPlayer::on_radio_selected), FRONT));
    vbuttonbox->pack_end (*radio_fb, false, false, 1);
                                                                                
    radio_bb = manage (new Gtk::RadioButton (group, "Ask back"));
    radio_bb->signal_clicked ().connect (
        sigc::bind<SideSelection>(
            sigc::mem_fun (*this, &DeckPlayer::on_radio_selected), BACK));
    vbuttonbox->pack_end (*radio_bb, false, false, 1);
	vbox->pack_start (*vbuttonbox, false, false, 0);

	if (m_is_real_deck) {
		m_selected_side = CONFIG->get_flipside_history (m_deck.get_name ());
		if (m_selected_side == FRONT) {
			radio_fb->set_active ();
		}
		else {
			radio_bb->set_active ();
		}
	}

	/** side-panel buttons
	 */
	m_shuffle_button->signal_clicked ().connect (
		sigc::mem_fun (*this, &DeckPlayer::on_shuffle_clicked));
	vbox->pack_start (*m_shuffle_button, false, false, 0);

	m_edit_button->signal_clicked ().connect (
		sigc::mem_fun (*this, &DeckPlayer::on_edit_clicked));
	vbox->pack_start (*m_edit_button, false, false, 0);

	m_count_label.set_alignment(0.5, 0.5);
    m_count_label.set_padding(0, 0);
	m_count_label.set_justify(Gtk::JUSTIFY_FILL);

	vbox->pack_end (*m_play_progress, false, false, 4);
	vbox->pack_end (m_count_label, false, false, 4);
	
	hbox->pack_start (*vbox, Gtk::PACK_SHRINK, 5);
	get_vbox ()->pack_start (*hbox);
	hbox->show_all ();

	/** Bottom playback controls toolbar panel
	 */
	m_playit_xpm = Gdk::Pixbuf::create_from_file (
		                               GRAPPDATADIR "/pixmaps/speaker_24.xpm");

	Glib::RefPtr<Gtk::IconFactory> icon_factory = Gtk::IconFactory::create ();
	Gtk::IconSet spkr_set;
	Gtk::IconSource spkr_source;
	spkr_source.set_pixbuf (m_playit_xpm);
	spkr_source.set_size (Gtk::ICON_SIZE_BUTTON);
	spkr_set.add_source (spkr_source);
	icon_factory->add (DeckPlayer::SAY_IT, spkr_set);
	Gtk::Stock::add (Gtk::StockItem (DeckPlayer::SAY_IT, "Say It!"));
	icon_factory->add_default ();

	m_actgroup = Gtk::ActionGroup::create ();

	m_actgroup->add (Gtk::Action::create ("Say_It", DeckPlayer::SAY_IT),
					 sigc::mem_fun (*this, &DeckPlayer::on_sayit_clicked));

	m_actgroup->add (Gtk::Action::create ("First", Gtk::Stock::GOTO_FIRST),
					 sigc::mem_fun (*this, &DeckPlayer::go_first_cb));

	m_actgroup->add (Gtk::Action::create ("Prev", Gtk::Stock::GO_BACK),
					 sigc::mem_fun (*this, &DeckPlayer::go_prev_cb));

	m_actgroup->add (Gtk::Action::create ("Next", Gtk::Stock::GO_FORWARD),
					 sigc::mem_fun (*this, &DeckPlayer::go_next_cb));

	m_actgroup->add (Gtk::Action::create ("Last", Gtk::Stock::GOTO_LAST),
					 sigc::mem_fun (*this, &DeckPlayer::go_last_cb));

	m_actgroup->add (Gtk::Action::create ("IKnowIt", Gtk::Stock::DIALOG_INFO),
					 sigc::mem_fun (*this, &DeckPlayer::know_answer_cb));

	m_actgroup->add (Gtk::Action::create ("NoClue", 
										  Gtk::Stock::DIALOG_QUESTION),
					 sigc::mem_fun (*this, &DeckPlayer::no_clue_cb));

	m_uimanager = Gtk::UIManager::create ();
	m_uimanager->insert_action_group (m_actgroup);
	add_accel_group (m_uimanager->get_accel_group ());

	DL((GRAPP,"Creating UI info\n"));
	try {
		Glib::ustring ui_info =
			"<ui>"
			"  <toolbar name='Controls'>"
			"     <toolitem action='Say_It'/>"
			"     <separator name='1'/>"
			"     <toolitem action='First'/>"
			"     <toolitem action='Prev'/>"
			"     <toolitem action='Next'/>"
			"     <toolitem action='Last'/>"
			"     <separator name='2'/>"
			"     <toolitem action='IKnowIt'/>"
			"     <toolitem action='NoClue'/>"
			"  </toolbar>"
			"</ui>";
		m_uimanager->add_ui_from_string (ui_info);
	}
	catch (const Glib::Error& ex_) {
		DL((GRAPP,"Building menues failed (%s)\n"));
		std::cerr << "Failed: " << ex_.what ();
	}
	DL((GRAPP,"UI info created\n"));

	m_toolbar_go_first = m_uimanager->get_widget ("/Controls/First");
	m_toolbar_go_prev  = m_uimanager->get_widget ("/Controls/Prev");
	m_toolbar_go_next  = m_uimanager->get_widget ("/Controls/Next");
	m_toolbar_go_last  = m_uimanager->get_widget ("/Controls/Last");
	m_toolbar_know_it  = m_uimanager->get_widget ("/Controls/IKnowIt");
	m_toolbar_no_clue  = m_uimanager->get_widget ("/Controls/NoClue");

	DL((GRAPP,"Got widget references back\n"));

	/** Add play controls' bar and <Close> button
	 */
	Gtk::Widget* controls = m_uimanager->get_widget ("/Controls");
	Gtk::Toolbar* ctrl_toolbar = dynamic_cast<Gtk::Toolbar*> (controls);
	ctrl_toolbar->set_orientation (Gtk::ORIENTATION_HORIZONTAL);
	ctrl_toolbar->set_toolbar_style (Gtk::TOOLBAR_ICONS);
	ctrl_toolbar->set_tooltips (true);

	set_has_separator (false);
	hbox = manage (new Gtk::HBox);
	hbox->pack_start (*ctrl_toolbar, true, true, 2);
	hbox->pack_start (*m_close_button, false, false, 0);
	get_vbox ()->pack_start (*hbox, false, false, 0);
	hbox->show_all ();

	m_close_button->signal_clicked ().connect (
		sigc::mem_fun (*this, &DeckPlayer::on_close_clicked));
	
	set_sensitivity ();
	m_deck.reset_progress ();
	set_win_name ();
	show_deck ();
	switch_side ();
	show_all_children ();	

	DL((GRAPP,"Ctor done\n"));
}

DeckPlayer::
~DeckPlayer ()
{
	trace_with_mask("DeckPlayer::~DeckPlayer", GUITRACE);
}

//==============================================================================
//                              Utilities
//==============================================================================

void
DeckPlayer::
show_deck ()
{
    trace_with_mask("DeckPlayer::show_deck",GUITRACE);

	if (m_deck.empty ()) {
		DL((GRAPP,"The deck is empty!\n"));
		return;
	}
	reset ();
}

void
DeckPlayer::
reset ()
{
    trace_with_mask("DeckPlayer::reset",GUITRACE);

	m_deck_iter = m_deck.begin ();
	show_card ();
}

void
DeckPlayer::
show_card ()
{
    trace_with_mask("DeckPlayer::show_card",GUITRACE);

	Glib::RefPtr<Gtk::TextBuffer> text_ref = m_example.get_buffer ();

	if (m_deck_iter != m_deck.end ()) {
		m_front.set_markup ("<big>" + (*m_deck_iter)->get_question ()+"</big>");
		m_back.set_markup  ("<big>" + (*m_deck_iter)->get_answer () + "</big>");
		text_ref->set_text ((*m_deck_iter)->get_example ());
		Gtk::TextBuffer::iterator iter = text_ref->begin ();
		while (iter != text_ref->end ()) {
			if (iter.get_char () == '~') {
				DL((APP,"Found marker \"~\"\n"));
				Gtk::TextBuffer::iterator iter_next = iter;
				iter_next++;
				text_ref->apply_tag (m_marker, iter, iter_next);
			}
			iter++;
		}
	}
	else {
		m_front.set_text ("");
		m_back.set_text ("");
		text_ref->set_text ("");
	}
	adjust_count ();
}

void
DeckPlayer::
adjust_count ()
{
    trace_with_mask("DeckPlayer::adjust_count",GUITRACE);

	if (m_deck.empty ()) {
		DL((GRAPP,"The deck is empty!\n"));
		m_count_label.set_text ("0/0");
		return;
	}
	string pmsg;
	float pcnt_done;
	m_deck.get_progress (pcnt_done, pmsg, m_deck_iter - m_deck.begin ());
	m_count_label.set_text (pmsg);
	m_play_progress->set_fraction (pcnt_done);
}

void
DeckPlayer::
set_sensitivity ()
{
	trace_with_mask("DeckPlayer::set_sensitivity", GUITRACE);

	if (m_is_real_deck) {
		m_toolbar_know_it ->set_sensitive (false);
		m_toolbar_no_clue ->set_sensitive (false);
	}
	else {
		m_toolbar_go_first->set_sensitive (false);
		m_toolbar_go_prev ->set_sensitive (false);
		m_toolbar_go_next ->set_sensitive (false);
		m_toolbar_go_last ->set_sensitive (false);

		m_shuffle_button  ->set_sensitive (false);
	}
}

//==============================================================================
//                          Action Buttons callbacks
//==============================================================================
void
DeckPlayer::
go_last_cb ()
{
	if ((m_deck_iter + 1) != m_deck.end ()) {
		m_deck_iter = m_deck.end ();
		m_deck_iter--;
		show_card ();
	}
	switch_side ();
}

void
DeckPlayer::
go_first_cb ()
{
	if (m_deck_iter != m_deck.begin ()) {
		m_deck_iter = m_deck.begin ();
		show_card ();
	}
	switch_side ();
}

void
DeckPlayer::
go_prev_cb ()
{
	if (m_deck_iter != m_deck.begin ()) {
		m_deck_iter--;
		show_card ();
	}
	switch_side ();
}

void
DeckPlayer::
go_next_cb ()
{
    trace_with_mask("DeckPlayer::go_next_cb",GUITRACE);

	if (m_deck_iter != m_deck.end () &&	         // empty deck
		(m_deck_iter + 1) != m_deck.end ())      // at the last element
	{
		DL((GRAPP,"Go next card=>\n"));
		m_deck_iter++;
		show_card ();
	}
	/** We flip the side only if user has gone (hopefully)
	 *  through the entier deck.
	 */
	if (m_is_real_deck) {
		if (m_deck_iter + 1 == m_deck.end () && !m_is_lesson_learned) {	
			DL((GRAPP,"Reached the last card!\n"));
			CONFIG->set_flipside_history (m_deck.get_name (), 
									  m_selected_side == FRONT ? BACK : FRONT);
			m_is_lesson_learned = true;
		}
	}
	switch_side ();
}

/** This callback is activated only when learning the cards in the
 	Card Box. We are always at the beginning of the list, moving
	elements to the next box or in case of the last box, moving 
	them to the back of the same list. We proceed in this manner until 
	all cards in the box are exhausted.
*/
void
DeckPlayer::
know_answer_cb ()
{
    trace_with_mask("DeckPlayer::know_answer_cb",GUITRACE);

	if (m_deck_iter == m_deck.end ()) {	// empty deck
		DL((GRAPP,"The deck is empty!\n"));
		return;
	}
	int old_idx = m_deck.get_index ();
	DL((GRAPP,"Moving card \"%s\" : deck %d => %d\n",
		(*m_deck_iter)->get_question ().c_str (), old_idx+1, old_idx+2));

	CARDBOX->move_card_to_box (m_deck_iter, old_idx, old_idx + 1, 
						(dynamic_cast<CardRef*> (*m_deck_iter))->is_expired ());
	switch_side ();
}

/** Move the card to the first box or if already in the first box,
	move it to the back of the list.
*/
void
DeckPlayer::
no_clue_cb ()
{
    trace_with_mask("DeckPlayer::no_clue_cb",GUITRACE);

	if (m_deck_iter == m_deck.end ()) {	// empty deck
		DL((GRAPP,"The deck is empty!\n"));
		return;
	}
	int old_idx = m_deck.get_index ();
	DL((GRAPP,"Moving card \"%s\" : deck %d => 1\n",
		(*m_deck_iter)->get_question ().c_str (), old_idx+1));

	CARDBOX->move_card_to_box (m_deck_iter, old_idx, 0,
						(dynamic_cast<CardRef*> (*m_deck_iter))->is_expired ());
	switch_side ();
}

void
DeckPlayer::
on_close_clicked ()
{
    trace_with_mask("DeckPlayer::on_close_clicked",GUITRACE);
    hide ();
}

void
DeckPlayer::
on_radio_selected (SideSelection selected_)
{
    trace_with_mask("DeckPlayer::on_radio_selected",GUITRACE);
	m_selected_side = selected_;
	switch_side ();
	DL((GRAPP,"Selected side : %s\n",
		(m_selected_side == FRONT ? "FRONT" : "BACK")));
}

void
DeckPlayer::
on_sayit_clicked ()
{
    trace_with_mask("DeckPlayer::on_sayit_clicked",GUITRACE);
	if (m_deck_iter != m_deck.end ()) {
		m_deck.play_card (m_deck_iter);
	}
}

void
DeckPlayer::
on_shuffle_clicked ()
{
    trace_with_mask("DeckPlayer::on_shuffle_clicked",GUITRACE);

	if (m_is_real_deck) {
		random_shuffle (m_deck.begin (), m_deck.end ());
		show_deck ();
	}
}

void
DeckPlayer::
on_edit_clicked ()
{
    trace_with_mask("DeckPlayer::on_edit_clicked",GUITRACE);
	DL((GRAPP,"Bring up deck viewer dialog\n"));

	int idx = m_deck_iter == m_deck.end() ? -1 : (m_deck_iter - m_deck.begin());
	DeckView dv (m_deck, idx);
	dv.run ();

	DL((GRAPP,"Deck size after edit = %d cards\n", m_deck.size ()));
    /** 
	 * Update viewer - iterators might have become invalid.
	 */
	if (m_deck.size () == 0) {	// Last card removed - clean up
		m_deck_iter == m_deck.end ();
		show_card ();
		on_close_clicked ();
	}
	else {
		show_deck ();
	}
}

void
DeckPlayer::
set_win_name ()
{
	string title = m_deck.get_name ();
	string::size_type idx = title.rfind ('/');
    if (idx != string::npos) {
        title.replace (0, idx+1, "");
    }
	set_title ("Deck Player : " + title);
}
