// -*- c++ -*-
//------------------------------------------------------------------------------
// $Id: DeckPlayer.h,v 1.20 2004/07/08 04:22:50 vlg Exp $
//------------------------------------------------------------------------------
//                            DeckPlayer.h
//------------------------------------------------------------------------------
//  Copyright (c) 2004 by Vladislav Grinchenko 
//
//  This program is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License   
//  as published by the Free Software Foundation; either version  
//  2 of the License, or (at your option) any later version.      
//------------------------------------------------------------------------------
//
// Date   : Fri Feb  6 23:37:11 EST 2004
//
//------------------------------------------------------------------------------
#ifndef DECK_PLAYER_H
#define DECK_PLAYER_H

#ifdef HAVE_CONFIG_H
#    include "config.h"
#endif

#include <gdkmm/pixbuf.h>
#include <gdkmm/image.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/notebook.h>
#include <gtkmm/label.h>
#include <gtkmm/textview.h>
#include <gtkmm/uimanager.h>

#include "VDeck.h"

class DeckView;
class Gtk::ProgressBar;

class DeckPlayer : public Gtk::Dialog
{
public:
	DeckPlayer (VDeck& vdeck_);
	~DeckPlayer ();

	/** Called in response to pressing <Close> button
	 */
	void on_close_clicked ();

	/** User pressed <Say It> button to play WAV audio, if available.
	 */
	void on_sayit_clicked ();

	/** Shuffle the cards in the deck
	 */
	void on_shuffle_clicked ();

	/** User pressed <Edit> button to Add/Delete/Edit cards
	 */
	void on_edit_clicked ();

	// -*- Button Actions -*-
	void go_first_cb ();		// Go to the first card
	void go_prev_cb ();			// Go back one card
	void go_next_cb ();			// Go to the next card
	void go_last_cb ();			// Go to the last card
	void know_answer_cb ();		// I know the answer
	void no_clue_cb ();			// Have no clue about this one

	// -*- Radio Button Actions -*-
	void on_radio_selected (SideSelection selected_);

	/** Get the deck pointer for comparison
	 */
	VDeck* get_deck () const { return &m_deck; }

	/** Reset iterator/count to reflect new card(s) added to the deck.
		Also, point iterator to the first card and show it. This method
		should be called each time the VDeck's count is modified (card is
		being moved or deleted).
	*/
	void reset ();

	/** Extend Stock Icons with application-specific ones.
	 */
	static Gtk::StockID SAY_IT;
	
private:
	void show_deck       ();	// Show first card of the loaded deck
	void show_card       ();	// Display the card pointed to by the iterator
	void switch_side     ();	// Show the front/back side of the card
	void adjust_count    ();	// Adjust count label: current/total
	void set_sensitivity ();    // De-sensetize inappropriate controls
	void set_win_name    ();    // Set tile name

private:
	Gtk::Button*      m_close_button;
	Gtk::Button*      m_shuffle_button;
	Gtk::Button*      m_edit_button;
	Gtk::Label        m_count_label;
	SideSelection     m_selected_side;
	Gtk::Notebook*    m_notebook;
	Gtk::ProgressBar* m_play_progress;
	Gtk::Label        m_front;
	Gtk::Label        m_back;
	Gtk::TextView     m_example;

	Gtk::Widget*      m_toolbar_go_first;
	Gtk::Widget*      m_toolbar_go_prev;
	Gtk::Widget*      m_toolbar_go_next;
	Gtk::Widget*      m_toolbar_go_last;
	Gtk::Widget*      m_toolbar_know_it;
	Gtk::Widget*      m_toolbar_no_clue;

	Glib::RefPtr<Gdk::Pixbuf> m_playit_xpm;	// Sound icon
	Gtk::Image*       m_playit_icon;

	Gdk::Color        m_bg_white;
	Gdk::Color        m_bg_pink;
	Gdk::Color        m_bg_black;

	VDeck&            m_deck;
	VDeck::cardlist_iterator m_deck_iter; // Position in the Deck

	Glib::RefPtr<Gtk::TextBuffer::TagTable> m_tag_table;
	Glib::RefPtr<Gtk::TextBuffer::Tag> m_marker;

	bool m_is_lesson_learned;	// Have user gone through the deck?
	bool m_is_real_deck;		// Is it a real Deck we are dealing with?

	Glib::RefPtr<Gtk::UIManager>   m_uimanager;
	Glib::RefPtr<Gtk::ActionGroup> m_actgroup;
};

inline void
DeckPlayer::
switch_side ()
{
	m_notebook->set_current_page (m_selected_side);
}
#endif /* DECK_PLAYER_H */
