// -*- c++ -*-
//------------------------------------------------------------------------------
// $Id: DeckView.h,v 1.11 2004/05/24 03:50:23 vlg Exp $
//------------------------------------------------------------------------------
//                            DeckView.h
//------------------------------------------------------------------------------
//  Copyright (c) 2004 by Vladislav Grinchenko 
//
//  This program is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License   
//  as published by the Free Software Foundation; either version  
//  2 of the License, or (at your option) any later version.      
//------------------------------------------------------------------------------
//
// Date   : Fri Feb  6 23:37:11 EST 2004
//
//------------------------------------------------------------------------------
#ifndef DECK_VIEWER_H
#define DECK_VIEWER_H

#ifdef HAVE_CONFIG_H
#    include "config.h"
#endif

#include <gtkmm/messagedialog.h>
#include <gtkmm/entry.h>
#include <gtkmm/label.h>
#include <gtkmm/frame.h>
#include <gtkmm/button.h>
#include <gtkmm/box.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <gtkmm/scrolledwindow.h>

class VDeck;

class DeckView : public Gtk::Dialog
{
public:
	DeckView (VDeck& deck_, int selected_row_);
	~DeckView ();

	/** User selected a row in the cards view
	 */
	void on_card_selected ();

	/** Called in response to pressing <Close> button
	 */
	void on_close_clicked () { /* no-op */ }

	// -*- Sidebar action button callbacks -*-
	void on_add_clicked         ();
	void on_edit_clicked        ();
	void on_delete_clicked      ();
	void on_more_clicked        ();
	void on_card_up_clicked     ();
	void on_card_down_clicked   ();
	void on_add_card_to_cardbox ();

private:
	void set_cards_count ();
	void set_win_name ();

private:
	typedef Gtk::TreeModel::Children children_t;
	typedef children_t::iterator chiter_t;

	struct ModelColumns : public Gtk::TreeModelColumnRecord
	{
		Gtk::TreeModelColumn<Glib::ustring> m_front_row;
		Gtk::TreeModelColumn<Glib::ustring> m_back_row;
		
		ModelColumns () { add (m_front_row); add (m_back_row); }
	};

	void autoscroll_up ();
	void autoscroll_down ();

	const ModelColumns   m_columns;
	Gtk::TreeView        m_tree_view;
	Gtk::ScrolledWindow* m_scrollwin;

	/// TreeModel
	Glib::RefPtr<Gtk::ListStore> m_list_store_ref; 

    /// Selection in current TreeView
	Glib::RefPtr<Gtk::TreeSelection> m_tree_sel_ref; 

	/** De-activate action buttons depending on the type 
	    of the underlying VDeck object.
	*/
	void set_sensitivity ();

private:
	Gtk::Button* m_add_button;
	Gtk::Button* m_edit_button;
	Gtk::Button* m_delete_button;
	Gtk::Button* m_close_button;
	Gtk::Button* m_more_button;
	Gtk::Button* m_card_up_button;
	Gtk::Button* m_card_down_button;
	Gtk::Button* m_add_card_to_cardbox;

	Gtk::Label*  m_label_count;

	VDeck& m_deck;
	int    m_row_idx;			// The row selected in the model
};

#endif /* DECK_VIEWER_H */
