// -*- c++ -*-
// Generated by assa-genesis
//------------------------------------------------------------------------------
// $Id: Granule.cpp,v 1.11 2004/07/22 12:10:04 vlg Exp $
//------------------------------------------------------------------------------
//                            Granule.cpp
//------------------------------------------------------------------------------
//  Copyright (c) 2004 by Vladislav Grinchenko 
//
//  This program is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License   
//  as published by the Free Software Foundation; either version  
//  2 of the License, or (at your option) any later version.      
//------------------------------------------------------------------------------
//
// Date   : Wed Dec 31 23:18:34 2003
//
//------------------------------------------------------------------------------

#ifdef HAVE_CONFIG_H
#   include "config.h"
#endif

#include <iostream>
#include <fstream>
#include <unistd.h>				// access(2)
#include <stdlib.h>				// system(3)

#include <gtkmm/main.h>

#include <assa/CommonUtils.h>
#include <assa/Logger.h>
#include <assa/SigAction.h>
#include <assa/IniFile.h>

#include "Granule-main.h"
#include "Granule.h"
#include "Config.h"
#include "MainWindow.h"

// Static declarations mandated by Singleton class

Granule* ASSA::Singleton<Granule>::m_instance;
ASSA::Destroyer<Granule> ASSA::Singleton<Granule>::m_destroyer;

static const int TIMEOUT = 500;

/*******************************************************************************
                          Member Functions
*******************************************************************************/
Granule::
Granule () :
	m_timeout (0.01),
	m_kit (NULL),
	m_main_window (NULL),
	m_secs_in_day (86400),
	m_secs_in_week (604800)
{
    add_opt ('g', "gtk-options",  &m_gtk_options );
	add_opt (0,   "secs-in-day",  &m_secs_in_day );
	add_opt (0,   "secs-in-week", &m_secs_in_week);

    // ---Configuration---
    rm_opt ('f', "config-file"  );
    rm_opt ('n', "instance"     );
    rm_opt ('p', "port"         );

    // ---Process bookkeeping---
    rm_opt ('b', "daemon"       );
    rm_opt ('l', "pidfile"      );
    rm_opt ('L', "ommit-pidfile");
	m_ommit_pidflock_flag = 0;	// don't lock up on PID 

    /*---
     * By defauil disable all debugging
     *---*/

    /* m_debug_mask = GRAPP | DECK | GUITRACE | ASSA::ERROR; */
	m_debug_mask = 0;

	/* m_with_log_server = true; */
    m_log_file = "granule.log";

}

/**
 * Description: 
 *
 *  Read configuration from [options] secion of the configuration
 *  file, ~/.granule, and populate command-line options. 
 *  They might be overwritten by command-line arguments supplied with argv_.
 *  Call GenServer::init() to start initialization.
 *
 *  If the application configuration file, ~/.granule,
 *  is missing, copy the default from $GRAPPDATADIR.
 *  Gtkmm hasn't been initialized yet.
 */
void 
Granule::
init (int* argc_, char* argv_[], const char* help_info_)
{
	string defaultrc (GRAPPDATADIR);
	defaultrc += "/granule";
	string rcfile (ASSA::Utils::strenv ("~/.granule"));

	ASSA::IniFile cfg (rcfile);
	std::string v;

	if (::access (rcfile.c_str (), R_OK) < 0) {
		if (::access (defaultrc.c_str (), R_OK) == 0) {
			string cmd ("cp -p ");
			cmd += defaultrc + " " + rcfile;
			if (::system (cmd.c_str ()) < 0) {
				goto done;
			}
		}
		else {
			goto done;
		}
	}

	if (cfg.load () == 0) {
		if (cfg.find_section ("options") != cfg.sect_end ()) {
			m_gtk_options     = cfg.get_value ("options", "gtk_options");

			if ((v = cfg.get_value ("options", "log_file")) != "") {
				m_log_file = v;
			}
			if ((v = cfg.get_value ("options","log_stdout")) != "") {
				m_log_stdout_flag = (v == "1" || v == "yes") ? true : false;
			}
			if ((v = cfg.get_value ("options", "log_size")) != "") {
				m_log_size = strtol (v.c_str (), NULL, 16); ;
			}
			if ((v = cfg.get_value ("options", "mask")) != "") {
				m_debug_mask = strtol (v.c_str (), NULL, 16);
			}
			if ((v = cfg.get_value ("options","log_level")) != "") {
				m_log_level = strtol (v.c_str(), NULL, 16);
			}
		}
	}

 done:

	ASSA::GenServer::init (argc_, argv_, help_info_);

	/*
	  TRACE    = 0x00001
	  APP      = 0x00002 (GRAPP)
	  USR1     = 0x00004 (GUITRACE)
	  USR2     = 0x00008 (DECK)
	  USR3     = 0x00010
	  ALL_APPS = 0x0001F
	  ERROR    = 0x00020
	  FORK     = 0x40000
	*/

	if (m_log_level >= 0) {
		switch (m_log_level) {
		case 5:	m_debug_mask = ALL;     break;    // Log message flood!
		case 4: m_debug_mask = 0x40026;	break;    // +FORK
		case 3: m_debug_mask = 0x26;    break;    // GRAPP+GUITRACE+ERRORS
		case 2: m_debug_mask = 0x25;    break;    // TRACE+GUITRACE+ERRORS
		case 1:	m_debug_mask = 0x22;    break;    // GRAPP+ERRORS
		case 0:	default: m_debug_mask = 0x0;
		}
	}

	LOGGER->enable_groups (m_debug_mask);
}

Granule::
~Granule ()
{
    trace_with_mask("Granule::~Granule",GUITRACE);
    delete m_main_window;
}

void
Granule::
dump_package_envs ()
{
	DL((APP,"\n= Environment variables set by Autoconfig: =\n"
		"\tGRAPPDATADIR = \"%s\"\n"
		"\tDATADIR     = \"%s\"\n"
		"\tPACKAGE     = \"%s\"\n"
		"\tVERSION     = \"%s\"\n\n",
		GRAPPDATADIR, DATADIR, PACKAGE, VERSION));
}

void
Granule::
initServer ()
{
    trace_with_mask("Granule::initServer",GUITRACE);

    ASSA::Log::disable_timestamp ();
    int gtk_argc = 0;
    char** gtk_argv = NULL;

	DL((ALL,"gtk_options = \"%s\"\n",     m_gtk_options.c_str ()));
	DL((ALL,"log_stdout  = %s\n",     m_log_stdout_flag ? "true" : "false"));
	DL((ALL,"log_file    = \"%s\"\n",     m_log_file.c_str ()));
	DL((ALL,"log_size    = %d\n",         m_log_size));
	DL((ALL,"log_level   = \"%d\"\n",     m_log_level));
	DL((ALL,"mask        = 0x%X\n",       m_debug_mask));

	/**
	 * Block SIGCHLD for a moment
	 */
	ASSA::SigAction old_chld_act;
	struct sigaction* sa;
	old_chld_act.retrieve_action (SIGCHLD);
	sa = old_chld_act;
	
    m_gtk_options = "granule --g-fatal-warnings " + m_gtk_options;
    CmdLineOpts::str_to_argv (m_gtk_options, gtk_argc, gtk_argv);

    m_kit = new Gtk::Main (&gtk_argc, &gtk_argv);

    CmdLineOpts::free_argv (gtk_argv);

	/** I suspect one of the Gnome libraries changes action of
	 *  SIGCHLD signal. Reset it to SIG_IGN.
	 */
	ASSA::SigAction ignore (SIG_IGN);
	ignore.register_action (SIGCHLD, &old_chld_act);

    /** Load history from ~/.granule configuration file
	 */
    CONFIG->load_config (m_secs_in_day, m_secs_in_week);

	/** Put up the main window on the screen.
	 */
    m_main_window = new MainWindow;
	m_main_window->init ();

    /** Mine (original) - core dumps on exit.
	 */
	sigc::slot0<bool> tslot = sigc::mem_fun (*this, &Granule::timer_cb);
    m_tconn = Glib::signal_timeout().connect (tslot, TIMEOUT);

	dump_package_envs ();
    DL((ASSA::APP,"Service has been initialized\n"));
}

void
Granule::
processServer ()
{
    trace_with_mask("Granule::processServer",GUITRACE);

    m_kit->run (*m_main_window);
	REACTOR->stopReactor ();

    DL((ASSA::APP,"Service stopped!\n"));
	gtk_exit (0);
}

