// -*- c++ -*-
// Generated by assa-genesis
//------------------------------------------------------------------------------
// $Id: MainWindow.cpp,v 1.30 2004/07/22 12:10:04 vlg Exp $
//------------------------------------------------------------------------------
//                            MainWindow.cpp
//------------------------------------------------------------------------------
//  Copyright (c) 2004 by Vladislav Grinchenko
//
//  This program is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License   
//  as published by the Free Software Foundation; either version  
//  2 of the License, or (at your option) any later version.      
//------------------------------------------------------------------------------
//
// Date   : Wed Dec 31 23:18:34 2003
//
//------------------------------------------------------------------------------

#include <gtkmm/stock.h>
#include <gtkmm/menu.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/filechooserdialog.h>

#include "Granule-main.h"
#include "Granule.h"
#include "MainWindow.h"
#include "Config.h"
#include "DeckList.h"
#include "CardBox.h"
#include "PrefWindow.h"
#include "AboutGRAPP.h"

//==============================================================================
//                          Static locals
//==============================================================================

const char* MainWindow::states [] = { "Start", "Dirty", "Clean", "Finish" };

static void 
destroy_handler ()
{
	trace_with_mask("[static] destroy_handler",GUITRACE);

	/* From Gtk 1.2 man pages:
	 *  "Makes the innermost invocation of the main loop return when it 
	 *   regains control."
	 */
    Gtk::Main::quit ();
}

//==============================================================================

MainWindow::
MainWindow () :
	m_state (Start),
	m_top_menu_bar (*this, m_deck_manager),
	m_open_item (NULL),
	m_app_title (PACKAGE),
	m_pref_window (NULL),
	m_about (NULL)
{
    trace_with_mask("MainWindow::MainWindow",GUITRACE);

    char pid [80];
    m_app_title += "-" + string (VERSION) + " (PID ";
    sprintf (pid, "%d", getpid ());
    m_app_title += pid + string (") ");

    add (m_contents);
    set_title (m_app_title.c_str ());
	set_resizable ();
    set_size_request (430, 350); // width; height
    set_border_width (2);
	set_position (Gtk::WIN_POS_CENTER_ALWAYS);

	/** Menu & Toolbar
	 */
    install_menus_and_toolbar ();

	/** Here go the guts of the main window (6 - spacing between cells).
	 */
	Gtk::HBox* hbox = manage (new Gtk::HBox (false, 6));

	Gtk::Frame* frame = manage (new Gtk::Frame);
	frame->set_label ("CardBox");
	m_cardbox = manage (new CardBox);
	frame->add (*m_cardbox);
	hbox->pack_start (*frame, Gtk::PACK_SHRINK);

	frame = manage (new Gtk::Frame);
	frame->set_border_width (8);
	m_deck_list = manage (new DeckList);

	m_top_menu_bar.set_deck_sensitivity (false, m_deck_list->size ());
	frame->add (*m_deck_list);
	hbox->pack_end (*frame);
	m_contents.pack_start (*hbox, Gtk::PACK_SHRINK);
	m_top_menu_bar.set_sensitivity (m_state);

    show_all ();
}


MainWindow::
~MainWindow ()
{
    trace_with_mask("MainWindow::~MainWindow",GUITRACE);

	if (m_pref_window != NULL) {
		delete m_pref_window;
		m_pref_window = NULL;
	}

    if (m_about != NULL) {
        delete m_about;
        m_about = NULL;
    }
}

void
MainWindow::
init ()
{
    trace_with_mask("MainWindow::init",GUITRACE);

	m_deck_list->init_popup_menu ();
}

void
MainWindow::
about_cb ()
{
    if (m_about == NULL) {
        m_about = new AboutGRAPP;
        m_about->set_transient_for (*this);
    }
    m_about->run ();
}

void 
MainWindow::
install_menus_and_toolbar ()
{
    trace_with_mask("MainWindow::install_menus_and_toolbar",GUITRACE);

	/** Create top menu bar 
	 *  ---------------------------------------
	 *   [File]                          [Help]
	 *  ---------------------------------------
	 */
	m_top_menu_bar.create ();
    m_contents.pack_start (*m_top_menu_bar.get_menu_bar (), Gtk::PACK_SHRINK);
}

int
MainWindow::
delete_event_impl (GdkEventAny* /*event_*/)
{
    trace_with_mask("MainWindow::delete_event_impl",GUITRACE);
	GRANULE->setStopServerFlag ();
    return false;
}

//==============================================================================
// Callbacks (external events)
//==============================================================================
void
MainWindow::
new_cb ()
{
    trace_with_mask("MainWindow::new_cb",GUITRACE);

    switch (m_state)
	{
	case Start:	 
		m_cardbox->new_file ();
		go_state (Dirty); 
		break;

	case Dirty:  /* ignored */    
	{
		Gtk::MessageDialog e1 ("Project has been modified.\n"
							   "Please, save and close it first",
							   MESSAGE_ERROR);
		e1.run ();
	}
	break;

	case Clean:  /* ignored */    
	{
		Gtk::MessageDialog e2 ("Project is opened.\n"
							   "Please, close it first", MESSAGE_ERROR);
		e2.run ();
	}
	break;

	case Finish: /* ignored */    break;
	}
    print_state ();
}

void
MainWindow::
open_cb ()
{
    trace_with_mask("MainWindow::open_cb",GUITRACE);
    
    // Switch the state
    switch (m_state) {

	case Start:	
		go_state (Clean);
		if (open_file () < 0) {
			go_state (Start);
		}
		m_top_menu_bar.add_history ();
		break;

	case Dirty:  /* ignored */
	{
		Gtk::MessageDialog emsg ("Project has been modified\n"
								 "Please, save and close it first",
								 MESSAGE_ERROR);
		emsg.run ();
	}
	break;

	case Clean:  /* ignored */ 
	{
		Gtk::MessageDialog emsg2 ("Project is opened\n"
								  "Please, close it first",
								  MESSAGE_ERROR);
		emsg2.run ();
	}
	break;

	case Finish: /* ignored */    
		break;
	}
    print_state ();
}

void
MainWindow::
open_recent_cb (guint index_)
{
    trace_with_mask("MainWindow::open_recent_cb",GUITRACE);

    // Switch the state
    switch (m_state) {
	case Start:	
	{
		go_state (Clean); 
		const char* doc = 
			(CONFIG->get_document_history () [index_]).c_str ();
		DL((TRACE,"Opening [%d] \"%s\"\n", index_, doc));
		if (m_cardbox->load (doc) < 0) {
			DL((TRACE,"Opening CardFile failed - removing from history\n"));
			Gtk::MessageDialog emsg0 ("CardFile you selected cannot be opened!",
									  Gtk::MESSAGE_ERROR);
			emsg0.run ();
			CONFIG->remove_document_history (doc);
			m_top_menu_bar.refill_history_list ();
			go_state (Start);
		}
		break;
	}
	case Dirty:  /* ignored */    
	{
		Gtk::MessageDialog emsg1 ("Project has been modified.\n"
								  "Please, save and close it first",
								  MESSAGE_ERROR);
		emsg1.run ();
	}
	break;
	case Clean:  /* ignored */    
	{
		Gtk::MessageDialog emsg2 ("Project is opened.\n"
								  "Please, close it first",
								  MESSAGE_ERROR);
		emsg2.run ();
	}
	break;
	case Finish: /* ignored */    break;
	}
    print_state ();
}

void
MainWindow::
save_cb ()
{
    trace_with_mask("MainWindow::save_cb",GUITRACE);

    switch (m_state) {
    case Start:	 /* ignored */    
	{
		Gtk::MessageDialog e1 ("Please open or create\nproject first",
							   MESSAGE_ERROR);
		e1.run ();
	}
	break;

    case Dirty:  
		save_file (); 
		go_state (Clean); 
		set_mw_name ();
		break;

    case Clean:  /* ignored */    
	{
		Gtk::MessageDialog e2 ("Project is not modified.\n"
							   "Close it to proceed.", MESSAGE_ERROR);
		e2.run ();
	}
	break;

    case Finish: /* ignored */    break;
    }
    print_state ();
}
void
MainWindow::
save_as_cb ()
{
    trace_with_mask("MainWindow::save_as_cb",GUITRACE);

    switch (m_state)
	{
	case Start:	 /* ignored */    
	{
		Gtk::MessageDialog emsg ("Please open or create\nproject first",
								 MESSAGE_ERROR);
		emsg.run ();
	}
	break;

	case Dirty:  
		go_state (Clean); 

	case Clean:  
		save_file_as (); 
		set_mw_name ();
		break;

	case Finish: /* ignored */    break;
	}
    print_state ();
}

void
MainWindow::
close_cb ()
{
    trace_with_mask("MainWindow::close_cb",GUITRACE);
    // Switch the state

    switch (m_state) {
    case Start:	 /* ignored */ 	break;
		/** 
		 * No 'break' in Dirty state, for the transition Clean->Start
		 * is an internal unconditional event.
		 */
    case Dirty:  
		go_state (Clean); 
		save_file (); 

    case Clean:  
		go_state (Start); 
		m_top_menu_bar.add_history ();
		set_title (m_app_title.c_str ()); // clear up title of old project name
		m_cardbox->close ();
		break;

    case Finish: /* ignored */ 	break;
    }
    print_state ();
}

void
MainWindow::
exit_cb ()
{
    trace_with_mask("MainWindow::exit_cb",GUITRACE);

    switch (m_state)
	{
	case Dirty:
	{
		go_state (Clean); 
		save_file (); 
		m_top_menu_bar.add_history ();
	}
	case Start:
	case Clean:
		CONFIG->dont_save_project ();
		DECKMGR->save_all_deck_cb ();
		CONFIG->save_config ();
		destroy_handler (); 
		go_state (Finish); 
		break;

	case Finish: /* ignored */ break;
	}
    print_state ();
}

void 
MainWindow::
pref_cb ()
{
    trace_with_mask("MainWindow::pref_cb",GUITRACE);

	change_preferences (); 
}

void 
MainWindow::
unselect_deck_cb ()
{
    trace_with_mask("MainWindow::unselect_deck_cb",GUITRACE);

	if (m_deck_list) {
		m_deck_list->unselect ();
		m_top_menu_bar.set_deck_sensitivity (false, m_deck_list->size ());
	}
}

//==============================================================================
//                          Internal Actions
//==============================================================================
int
MainWindow::
open_file ()
{
    trace_with_mask("MainWindow::open_file",GUITRACE);

	Gtk::FileChooserDialog f ("Select Card File", 
							  Gtk::FILE_CHOOSER_ACTION_OPEN);
	f.add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	f.add_button ("Select",           Gtk::RESPONSE_OK);

	f.set_current_folder (CONFIG->get_filesel_path ());
	f.set_local_only ();

	Gtk::FileFilter filter;
	filter.set_name ("CardFile");
	filter.add_pattern ("*" CARD_FILE_EXT);
	f.set_filter (filter);
	if (f.run () != Gtk::RESPONSE_OK) {
		return -1;
	}
	
	return (m_cardbox->load (f.get_filename ())); 
}

int
MainWindow::
save_file ()
{
    trace_with_mask("MainWindow::save",GUITRACE);

	string projname = CONFIG->get_proj_name ();
	if (projname == UNKNOWN_FNAME) {
		Gtk::FileSelection f ("Choose New CardFile name");
		f.set_filename (CONFIG->get_filesel_path ());
		f.complete ("*" CARD_FILE_EXT);
		f.hide_fileop_buttons ();

		if (f.run () != Gtk::RESPONSE_OK) {
			return -1;
		}
		m_cardbox->save_as (f.get_filename ());
	}
	else {
		m_cardbox->save ();
	}
	return 0;
}

/**
 * Set window title
 */
void
MainWindow::
set_mw_name (void) 
{
    trace_with_mask("MainWindow::set_mw_name",GUITRACE);

    string title (m_app_title);
    string p (CONFIG->get_proj_name ());
    DL((TRACE,"project_name = \"%s\"\n",p.c_str ()));
    if (p.size () != 0) {
		title = p + " : " + title;
    }
    set_title (title.c_str ());
}

void
MainWindow::
save_file_as ()
{
    trace_with_mask("MainWindow::save_file_as",GUITRACE);

	Gtk::FileChooserDialog f ("Save Card File As ...", 
							  Gtk::FILE_CHOOSER_ACTION_SAVE);
	f.add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	f.add_button ("Save",           Gtk::RESPONSE_OK);

	f.set_current_folder (CONFIG->get_filesel_path ());
	f.set_local_only ();

	Gtk::FileFilter filter_cdf;
	filter_cdf.set_name ("CardFile");
	filter_cdf.add_pattern ("*" CARD_FILE_EXT);
	f.set_filter (filter_cdf);

	if (f.run () != Gtk::RESPONSE_OK) {
		return;
	}
	m_cardbox->save_as (f.get_filename ());
}

/**
 * Return: true if preference(s) has been changed; false otherwise
 */
bool
MainWindow::
change_preferences ()
{
    trace_with_mask("MainWindow::change_preferences",GUITRACE);

    if (m_pref_window == NULL) {
		m_pref_window = new PrefWindow;
    }
    
    bool ret = m_pref_window->run ();
    DL((GRAPP,"Preferences %s\n", ret ? "changed" : "unchanged"));
    return ret;
}

//==============================================================================
// Utilities
//==============================================================================

void
MainWindow::
play_in_box (int idx_)
{
    trace_with_mask("MainWindow::play_in_box",GUITRACE);
	DL((GRAPP,"Starting DeckPlayer for box # %d\n", idx_));
	if (m_cardbox) {
		m_cardbox->play_deck (idx_);
	}
}

inline void
MainWindow::
print_state (void) const
{
    DL((APP,"Current state: \"%s\"\n", states [m_state]));
}

inline void
MainWindow::
go_state (WMState state_)
{
    DL((APP,"Changing state \"%s\" => \"%s\"\n", 
		states [m_state], states [state_]));
    m_state = state_;
    m_top_menu_bar.set_sensitivity (state_);
}
